@@ -110,17 +110,17 @@
  **                   util.c.
  **                 * Reecriture du Makefile
  **                 * Modification de conf.h pour les variables globales
- ** 				* Changement dans les logs : utilise syslog(3) _ou_ un
- ** 			      fichier de log annexe, pas les deux.
+ **                 * Changement dans les logs : utilise syslog(3) _ou_ un
+ **                   fichier de log annexe, pas les deux.
  ** 2.4 07/01/94    * FreeBSD a la fonction initgroups(2)
  **                 * FreeBSD 1.1 n'a pas besoin du hack violent pour tcsh
  **                 * correction bug dans lequel wanted_user est ecrasee
- ** 				  par l'appel a getpwname(calife).
- ** 				* FreeBSD 1.1 dispose des YP !!
- ** 				* Changement de _POSIX_SOURCE en POSIX_SIGNALS
+ **                   par l'appel a getpwname(calife).
+ ** 	              * FreeBSD 1.1 dispose des YP !!
+ ** 	              * Changement de _POSIX_SOURCE en POSIX_SIGNALS
  ** 2.5 22/04/94    * Incorporation de modifs pour SVR4 et sa p... de gestion
- ** 				  des shadow passwords.
- ** 				* Securisation du programme en empechant les cores avec
+ **                   des shadow passwords.
+ **                 * Securisation du programme en empechant les cores avec
  **                   rlimit et la gestion des uid/euid/ssuid. Nouvelle option
  **                   HAVE_RLIMIT.
  **                 * Modification de la gestion de wait(2) : ajout du #define
@@ -175,19 +175,22 @@
  **                 * il est possible de spcifier un group dans calife.auth
  **                   via @group:...
  **                 * syslog(3) est maintenant requis.
+ ** 3.0 04/05/05    * PAMification de calife, PAM est dsormais *obligatoire*
+ **                   les gens ne l'ayant pas sont invits  rester  2.8.
+ **                 * Le mode RELAXED sans mot de passe disparait.
  **/
 
 #define MAIN_MODULE
 
+#ifndef lint
+static const char * rcsid = "@(#) $Id$";
+#endif
+
 #include "config.h"     /* gnr par configure */
 
                         /* fichier de configuration */
 #include "conf.h"
 
-#ifndef lint
-static char * rcsid = "$Id: //depot/security/calife/main/calife.c#45 $";
-#endif /* lint */
-
 FILE    * fp = NULL, * logfile = NULL;  /* fichier d'auth. et log */
 int     custom_shell= 0;            /* modification du shell ? */
 char    * shell;                    /* nom du shell */
@@ -195,19 +198,16 @@ uid_t   ssid;   					/* saved uid -- POSIX */
 int     want_login = 0;             /* like 'su -' or not */
 char    * _group;                   /* if auth. through group */
 
+#ifdef WITH_PAM
+pam_handle_t	*pamh = NULL;
+#endif /* WITH_PAM */
+
 #ifdef HAVE_RLIMIT
 static struct rlimit rlp, orlp;
 #endif
 
-#ifdef STDC_HEADERS
 int 
 main (int argc, char * argv [])
-#else /* !STDC_HEADERS */
-int 
-main(argc, argv)
-    int    argc;
-    char * argv [];
-#endif
 {
     int             allowed = 0;        /* l'utilisateur est-il valide ? */
     char            * name, * user_to_be, * tty, this_time [30];
@@ -222,6 +222,9 @@ main(argc, argv)
 #ifdef HAVE_POSIX_SIGNALS
     sigset_t        old_set, sig_set;
 #endif
+#ifdef WITH_PAM
+    int             e;
+#endif
 
     name = NULL;
     tty = NULL;
@@ -236,28 +239,21 @@ main(argc, argv)
 
                         /* displays compilation options */
 #ifdef DEBUG
-#ifdef STDC_HEADERS
     fprintf (stderr, "Calife : How to become someone else legally\n"
                      "Distribute in respect of the GNU General Public Licence\n"
-                     "Copyright (c) 1991, 2002 by Ollivier ROBERT\n"
-                     "%s\n", rcsid);
-#else /* !STDC_HEADERS */
-    fprintf (stderr, "Calife : How to become someone else legally\n"
-    fprintf (stderr, "Distribute in respect of the GNU General Public Licence\n");
-    fprintf (stderr, "Copyright (c) 1991, 2002 by Ollivier ROBERT\n");
-    fprintf (stderr, "%s\n", rcsid);
-#endif /* STDC_HEADERS */
-    fprintf (stderr, "Options : ");
+                     "Copyright (c) 1991-2008 by Ollivier ROBERT\n");
+    fprintf (stderr, "Revision: %s", rcsid);
+    fprintf (stderr, "Options: ");
     fprintf (stderr, "custom_shell, ");
 #ifdef NO_SETUID_SHELL 
     fprintf (stderr, "no_setuid_shell, ");
 #endif /* NO_SETUID_SHELL */
-#ifdef RELAXED
-    fprintf (stderr, "relaxed_mode, ");
 #ifdef WANT_GLOBAL_RC
     fprintf(stderr, "global_rc, ");
 #endif /* WANT_GLOBAL_RC */
-#endif /* RELAXED */
+#ifdef WITH_PAM
+    fprintf (stderr, "with_pam,");
+#endif /* WITH_PAM */
     fprintf (stderr, "su_like, ");
     fprintf (stderr, "debug\n");
     fflush (stderr);
@@ -267,10 +263,12 @@ main(argc, argv)
     /*
      * save the old limit values
      */
+#ifndef DEBUG
     getrlimit (RLIMIT_CORE, &orlp);
     rlp.rlim_max = 0;   		/* no core file */
     rlp.rlim_cur = 0;
     setrlimit (RLIMIT_CORE, &rlp);
+#endif /* !DEBUG */
 #endif /* HAVE_RLIMIT */    
 #ifndef HAVE_POSIX_SIGNALS
                                 /* no signals !! */
@@ -463,11 +461,11 @@ main(argc, argv)
     /*
      * open syslod log file
      */
-#ifndef RELAXED
+#ifdef WITH_PAM
+    openlog ("calife/pam", LOG_PID | LOG_CONS, LOG_AUTH);
+#else
     openlog ("calife+", LOG_PID | LOG_CONS, LOG_AUTH);
-#else /* RELAXED */
-    openlog ("calife", LOG_PID | LOG_CONS, LOG_AUTH);
-#endif /* RELAXED */
+#endif /* WITH_PAM */
     /*
      * does the user we want to become exist or not ? 
      */
@@ -514,7 +512,7 @@ main(argc, argv)
         calife = (struct passwd *) xalloc (sizeof (struct passwd));
         wanted_user = (struct passwd *) xalloc (sizeof (struct passwd));
 
-        verify_password (name, user_to_be, this_time, tty);
+        authenticate_user (name, user_to_be, this_time, tty);
         switch (fork ())
         {
             case -1:               
@@ -530,7 +528,9 @@ main(argc, argv)
                 test_user = getpwnam (user_to_be);
                 memcpy (wanted_user, test_user, sizeof (struct passwd));
 #ifdef DEBUG
-                fprintf (stderr, "uid,gid: %lu,%lu --> %u,%u\n", getuid(), getgid(), wanted_user->pw_uid, wanted_user->pw_gid);
+                MESSAGE_4 ("uid,gid: %lu,%lu --> %u,%u\n", getuid(), \
+                           getgid(), wanted_user->pw_uid, \ 
+                           wanted_user->pw_gid);
 #endif             
 #ifdef HAVE_POSIX_SIGNALS
                 sigprocmask (SIG_SETMASK, &old_set, NULL);
@@ -589,6 +589,23 @@ main(argc, argv)
                 free (calife);
                 free (wanted_user);
 
+#ifdef WITH_PAM
+                	if (pamh) 
+                	{
+                		if ((e = pam_open_session(pamh, 0)) != PAM_SUCCESS) 
+                		{
+                			syslog (LOG_AUTH | LOG_ERR, "pam_open_session: %s",
+                			    pam_strerror (pamh, e));
+                		}
+                		else if ((e = pam_setcred(pamh, PAM_ESTABLISH_CRED)) !=
+                		    PAM_SUCCESS) 
+                		{
+                			syslog (LOG_AUTH | LOG_ERR, "pam_setcred: %s",
+                			    pam_strerror (pamh, e));
+                		}
+                	}
+#endif /* WITH_PAM */
+
                 if (allowed == 2)
                     syslog (LOG_AUTH | LOG_NOTICE, "%s@%s to %s on %s - BEGIN", 
                             name, _group, user_to_be, tty);
@@ -689,19 +706,36 @@ main(argc, argv)
                         fprintf (stderr, "\a%s not executable/readable.\n", out_rc);
                 }
                 done:                    
+                free (out_rc);
 #endif /* WANT_GLOBAL_RC */
                 if (allowed == 2)
                     syslog (LOG_AUTH | LOG_NOTICE, "%s@%s to %s on %s - END.", 
                             name, _group, user_to_be, tty);
                 else
                     syslog (LOG_AUTH | LOG_NOTICE, "%s to %s on %s - END.", name, user_to_be, tty);
-                closelog ();
                 /*
                  * cleanup
                  */
-#ifdef WANT_GLOBAL_RC
-                free (out_rc);
-#endif /* WANT_GLOBAL_RC */
+#ifdef WITH_PAM
+                if (pamh)
+                {
+                    if ((e = pam_setcred (pamh, PAM_DELETE_CRED)) != \
+                                          PAM_SUCCESS)
+                        syslog (LOG_AUTH | LOG_ERR, "pam_setcred: %s",\
+                                pam_strerror(pamh, e));
+
+                    if ((e = pam_close_session (pamh, 0)) != PAM_SUCCESS)
+                        syslog(LOG_AUTH | LOG_ERR, "pam_close_session: %s",\
+                               pam_strerror(pamh, e));
+
+                    if ((e = pam_end (pamh, e)) != PAM_SUCCESS)
+                        syslog(LOG_AUTH | LOG_ERR, "pam_end: %s",\
+                               pam_strerror(pamh, e));
+                    pamh = NULL;
+                }
+#endif
+                closelog ();
+
 #ifdef DEBUG
                 if (WIFEXITED (status))
                 {
@@ -753,14 +787,8 @@ main(argc, argv)
  ** Privileges :    aucun
  **/
 
-#ifdef STDC_HEADERS
 void 
 exec_shell (char * shell_name)
-#else /* !STDC_HEADERS */
-void 
-exec_shell (shell_name)
-char * shell_name;
-#endif /* STDC_HEADERS */
 {
     int         fd, err;
     char        * shell_arg0;
